/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(NETSCAPE_PLUGIN_API)
#include "NPObjectMessageReceiver.h"

#include "ArgumentCoders.h" // NOLINT
#include "Decoder.h" // NOLINT
#include "HandleMessage.h" // NOLINT
#include "NPIdentifierData.h" // NOLINT
#include "NPObjectMessageReceiverMessages.h" // NOLINT
#include "NPVariantData.h" // NOLINT
#include <wtf/Vector.h> // NOLINT

#if ENABLE(IPC_TESTING_API)
#include "JSIPCBinding.h"
#endif

namespace Messages {

namespace NPObjectMessageReceiver {

} // namespace NPObjectMessageReceiver

} // namespace Messages

namespace WebKit {

void NPObjectMessageReceiver::didReceiveNPObjectMessageReceiverMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, IPC::description(decoder.messageName()), decoder.destinationID());
}

bool NPObjectMessageReceiver::didReceiveSyncNPObjectMessageReceiverMessage(IPC::Connection& connection, IPC::Decoder& decoder, UniqueRef<IPC::Encoder>& replyEncoder)
{
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::Deallocate::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::Deallocate>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::deallocate);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::HasMethod::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::HasMethod>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::hasMethod);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::Invoke::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::Invoke>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::invoke);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::InvokeDefault::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::InvokeDefault>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::invokeDefault);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::HasProperty::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::HasProperty>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::hasProperty);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::GetProperty::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::GetProperty>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::getProperty);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::SetProperty::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::SetProperty>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::setProperty);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::RemoveProperty::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::RemoveProperty>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::removeProperty);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::Enumerate::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::Enumerate>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::enumerate);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::Construct::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::Construct>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::construct);
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    UNUSED_PARAM(replyEncoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return false;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled synchronous message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
    return false;
}

} // namespace WebKit

#if ENABLE(IPC_TESTING_API)

namespace IPC {

template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::NPObjectMessageReceiver_Deallocate>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::Deallocate::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessageReply<MessageName::NPObjectMessageReceiver_Deallocate>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::Deallocate::ReplyArguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::NPObjectMessageReceiver_HasMethod>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::HasMethod::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessageReply<MessageName::NPObjectMessageReceiver_HasMethod>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::HasMethod::ReplyArguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::NPObjectMessageReceiver_Invoke>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::Invoke::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessageReply<MessageName::NPObjectMessageReceiver_Invoke>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::Invoke::ReplyArguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::NPObjectMessageReceiver_InvokeDefault>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::InvokeDefault::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessageReply<MessageName::NPObjectMessageReceiver_InvokeDefault>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::InvokeDefault::ReplyArguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::NPObjectMessageReceiver_HasProperty>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::HasProperty::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessageReply<MessageName::NPObjectMessageReceiver_HasProperty>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::HasProperty::ReplyArguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::NPObjectMessageReceiver_GetProperty>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::GetProperty::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessageReply<MessageName::NPObjectMessageReceiver_GetProperty>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::GetProperty::ReplyArguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::NPObjectMessageReceiver_SetProperty>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::SetProperty::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessageReply<MessageName::NPObjectMessageReceiver_SetProperty>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::SetProperty::ReplyArguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::NPObjectMessageReceiver_RemoveProperty>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::RemoveProperty::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessageReply<MessageName::NPObjectMessageReceiver_RemoveProperty>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::RemoveProperty::ReplyArguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::NPObjectMessageReceiver_Enumerate>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::Enumerate::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessageReply<MessageName::NPObjectMessageReceiver_Enumerate>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::Enumerate::ReplyArguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::NPObjectMessageReceiver_Construct>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::Construct::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessageReply<MessageName::NPObjectMessageReceiver_Construct>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::NPObjectMessageReceiver::Construct::ReplyArguments>(globalObject, decoder);
}

}

#endif


#endif // ENABLE(NETSCAPE_PLUGIN_API)
