/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(NETSCAPE_PLUGIN_API)
#include "PluginProcess.h"

#include "ArgumentCoders.h" // NOLINT
#include "Decoder.h" // NOLINT
#include "HandleMessage.h" // NOLINT
#include "PluginProcessCreationParameters.h" // NOLINT
#include "PluginProcessMessages.h" // NOLINT
#include <wtf/Vector.h> // NOLINT
#include <wtf/WallTime.h> // NOLINT
#include <wtf/text/WTFString.h> // NOLINT

#if ENABLE(IPC_TESTING_API)
#include "JSIPCBinding.h"
#endif

namespace WebKit {

void PluginProcess::didReceivePluginProcessMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::PluginProcess::InitializePluginProcess::name())
        return IPC::handleMessage<Messages::PluginProcess::InitializePluginProcess>(connection, decoder, this, &PluginProcess::initializePluginProcess);
    if (decoder.messageName() == Messages::PluginProcess::CreateWebProcessConnection::name())
        return IPC::handleMessage<Messages::PluginProcess::CreateWebProcessConnection>(connection, decoder, this, &PluginProcess::createWebProcessConnection);
    if (decoder.messageName() == Messages::PluginProcess::GetSitesWithData::name())
        return IPC::handleMessage<Messages::PluginProcess::GetSitesWithData>(connection, decoder, this, &PluginProcess::getSitesWithData);
    if (decoder.messageName() == Messages::PluginProcess::DeleteWebsiteData::name())
        return IPC::handleMessage<Messages::PluginProcess::DeleteWebsiteData>(connection, decoder, this, &PluginProcess::deleteWebsiteData);
    if (decoder.messageName() == Messages::PluginProcess::DeleteWebsiteDataForHostNames::name())
        return IPC::handleMessage<Messages::PluginProcess::DeleteWebsiteDataForHostNames>(connection, decoder, this, &PluginProcess::deleteWebsiteDataForHostNames);
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::PluginProcess::SetQOS::name())
        return IPC::handleMessage<Messages::PluginProcess::SetQOS>(connection, decoder, this, &PluginProcess::setQOS);
#endif
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, IPC::description(decoder.messageName()), decoder.destinationID());
}

} // namespace WebKit

#if ENABLE(IPC_TESTING_API)

namespace IPC {

template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::PluginProcess_InitializePluginProcess>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::PluginProcess::InitializePluginProcess::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::PluginProcess_CreateWebProcessConnection>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::PluginProcess::CreateWebProcessConnection::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::PluginProcess_GetSitesWithData>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::PluginProcess::GetSitesWithData::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::PluginProcess_DeleteWebsiteData>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::PluginProcess::DeleteWebsiteData::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::PluginProcess_DeleteWebsiteDataForHostNames>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::PluginProcess::DeleteWebsiteDataForHostNames::Arguments>(globalObject, decoder);
}
#if PLATFORM(COCOA)
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::PluginProcess_SetQOS>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::PluginProcess::SetQOS::Arguments>(globalObject, decoder);
}
#endif

}

#endif


#endif // ENABLE(NETSCAPE_PLUGIN_API)
